#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Python command line application to convert Markdown to PDF.

.. currentmodule:: mdpdf
.. moduleauthor:: Norman Lorrain <normanlorrain@gmail.com>
"""

